from threading import Event
import ao, mad, time

debug = True

class Decoder:

  #

  bits   = 16
  rate   = 44100
  driver = "sun"

  filename = str()

  dsp     = None
  buf     = None
  madfile = None

  # if this is set, we need to stop ASAP
  stopBit    = Event()

  # we set this to show that we've stopped
  stoppedBit = Event()

  getLen        = lambda: self.madfile.total_time()/1000
  getBitrate    = lambda: self.madfile.bitrate()/1000
  getSampleRate = lambda: self.madfile.samplerate()

  #

  def __init__(self):
    if debug: print time.time(), "Decoder: __init__: running..."
    self.initAudio()
    if debug: print time.time(), "Decoder: __init__: done"

  def initAudio(self):
    if debug: print time.time(), "Decoder: initAudio: running..."
    self.dsp = ao.AudioDevice(self.driver, bits=self.bits, rate=self.rate)
    if debug: print time.time(), "Decoder: initAudio: done"

  def getBuf(self):
    self.buf = self.madfile.read()
    return self.buf and True
  
  def outputFrame(self):
    self.dsp.play(self.buf, len(self.buf))

  def seek(self, secs):
    return self.madfile.seek_time(secs*1000)

  def clearData(self):
    if debug: print time.time(), "Decoder: clearData: running..."
    self.madfile = None
    self.buf = None
    self.filename = str()
    if debug: print time.time(), "Decoder: clearData: done"

  def stop(self):
    if debug: print time.time(), "Decoder: stop: running"
    self.stopBit.clear()
    self.clearData()
    self.stoppedBit.set()
    if debug: print time.time(), "Decoder: stop: done"

  def play(self, filename):
    if debug: print time.time(), "Decoder: play: running..."
    if debug: print time.time(), "Decoder: play: file:", filename
    self.filename = filename
    self.madfile = mad.MadFile(self.filename)

    self.stoppedBit.clear()

    while self.getBuf():
      if not self.stopBit.isSet():
        self.outputFrame()
      else:
        if debug: print time.time(), "Decoder: play: stopBit set, stopping"
        self.stop()
        break
    else:
      self.clearData()

    if debug: print time.time(), "Decoder: play: done with song", filename
    if debug: print time.time(), "Decoder: play: returning"

